@@ -9,11 +9,10 @@ from asyncio import StreamReader, IncompleteReadError
 from collections import defaultdict
 from fnmatch import fnmatch
 
-from prometheus_client import start_http_server
-
 from livelock.shared import DEFAULT_RELEASE_ALL_TIMEOUT, DEFAULT_BIND_TO, DEFAULT_LIVELOCK_SERVER_PORT, get_settings, DEFAULT_MAX_PAYLOAD, pack_resp, DEFAULT_SHUTDOWN_SUPPORT, \
-    DEFAULT_PROMETHEUS_PORT
+    DEFAULT_PROMETHEUS_PORT, DEFAULT_TCP_KEEPALIVE_TIME, DEFAULT_TCP_KEEPALIVE_INTERVAL, DEFAULT_TCP_KEEPALIVE_PROBES, DEFAULT_LOGLEVEL, DEFAULT_DISABLE_DUMP_LOAD
 from livelock.stats import latency, max_lock_live_time
+from livelock.tcp_opts import set_tcp_keepalive
 
 ABSOLUTE_PATH = lambda x: os.path.abspath(os.path.join(os.path.abspath(os.path.dirname(__file__)), x))
 logger = logging.getLogger(__name__)
@@ -165,8 +164,8 @@ class StorageAdaptor(LockStorage):
             # Good time to dump and exit(), but there is chance that some client's will not get their responses
             self.dump_before_die()
 
-        # In case storage operations in progress we will dump data after last operations completes
-        # and will exit() when last reply is send
+        # In case any storage operations is in progress, we will dump data after last operations completes
+        # and will exit() when last reply is sent
         if not self._commands_in_process:
             self.die()
 
@@ -235,6 +234,8 @@ class StorageAdaptor(LockStorage):
     def clear_dump(self):
         return self.store.clear_dump()
 
+    def stats(self):
+        return self.store.stats()
 
 class InMemoryLockStorage(LockStorage):
     def __init__(self, *args, **kwargs):
@@ -292,7 +293,7 @@ class InMemoryLockStorage(LockStorage):
         return True
 
     def release_all(self, client_id, timeout=None):
-        mark_free_at = time.time() + self.release_all_timeout if not timeout else timeout
+        mark_free_at = time.time() + (self.release_all_timeout if not timeout else timeout)
         for lock in self.client_to_locks[client_id]:
             lock.mark_free_after = mark_free_at
         logger.debug(f'Marked to free at {mark_free_at} for {client_id}')
@@ -392,20 +393,45 @@ class InMemoryLockStorage(LockStorage):
 
     def clear_dump(self):
         if os.path.isfile(self._dump_file_name):
-            logger.debug('Removing in memory lock data file %s' % ABSOLUTE_PATH(self._dump_file_name))
+            logger.debug('Removing in memory lock data file %s' % os.path.abspath(self._dump_file_name))
             os.remove(self._dump_file_name)
 
     def stats(self):
-        return dict(lock_count=len(self.all_locks))
+        return dict(lock_count=len(self.all_locks), dump_file_path=os.path.abspath(self._dump_file_name))
 
 
 class CommandProtocol(asyncio.Protocol):
-    def __init__(self, adaptor, max_payload, *args, **kwargs):
+    def __init__(self, tcp_keepalive_time=None, tcp_keepalive_interval=None, tcp_keepalive_probes=None, *args, **kwargs):
         self.transport = None
-        self.max_payload = max_payload
         self._reader = None
 
-        super().__init__()
+        """
+        http://coryklein.com/tcp/2015/11/25/custom-configuration-of-tcp-socket-keep-alive-timeouts.html
+        Keep-Alive Process
+        There are three configurable properties that determine how Keep-Alives work. On Linux they are1:
+        
+        tcp_keepalive_time
+        default 7200 seconds
+        tcp_keepalive_probes
+        default 9
+        tcp_keepalive_intvl
+        default 75 seconds
+        The process works like this:
+        
+        1. Client opens TCP connection
+        2. If the connection is silent for tcp_keepalive_time seconds, send a single empty ACK packet.1
+        3. Did the server respond with a corresponding ACK of its own?
+            - No
+            1. Wait tcp_keepalive_intvl seconds, then send another ACK
+            2. Repeat until the number of ACK probes that have been sent equals tcp_keepalive_probes.
+            3. If no response has been received at this point, send a RST and terminate the connection.
+            - Yes: Return to step 2
+        """
+        self.tcp_keepalive_time = int(tcp_keepalive_time)
+        self.tcp_keepalive_interval = int(tcp_keepalive_interval)
+        self.tcp_keepalive_probes = int(tcp_keepalive_probes)
+
+        super().__init__(*args, **kwargs)
 
     def data_received(self, data):
         if self.kill_active:
@@ -415,6 +441,14 @@ class CommandProtocol(asyncio.Protocol):
     def connection_made(self, transport):
         if self.kill_active:
             return
+
+        sock = transport.get_extra_info('socket')
+        set_tcp_keepalive(sock, opts=dict(tcp_keepalive=True,
+                                          tcp_keepalive_idle=self.tcp_keepalive_time,
+                                          tcp_keepalive_intvl=self.tcp_keepalive_interval,
+                                          tcp_keepalive_cnt=self.tcp_keepalive_probes,
+                                          ))
+
         self.transport = transport
         self._reader = StreamReader()
         self._reader.set_transport(transport)
@@ -524,14 +558,17 @@ class LiveLockProtocol(CommandProtocol):
     def __init__(self, adaptor, password, max_payload, shutdown_support=False, *args, **kwargs):
         self.password = password
         self.adaptor = adaptor
+        self.max_payload = max_payload
+
         self.client_id = None
         self.client_info = None
         self.shutdown_support = shutdown_support
         self._authorized = None
         self.transport = None
+
         self.debug = get_settings(None, 'LIVELOCK_DEBUG', False)
 
-        super().__init__(adaptor=adaptor, max_payload=max_payload, *args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     @property
     def client_display(self):
@@ -710,6 +747,12 @@ class LiveLockProtocol(CommandProtocol):
                         return
                     result = list(self.adaptor.find(args[0].decode()))
                     self._reply_data(result)
+                elif verb == 'stats':
+                    if len(args):
+                        self._reply(WRONG_ARGS)
+                        return
+                    result = self.adaptor.stats()
+                    self._reply_data(result)
                 elif verb == 'shutdown' and self.shutdown_support:
                     logger.debug(f'SHUTDOWN invoked by {self.client_display}')
                     self.adaptor.terminate()
@@ -785,6 +828,7 @@ class StorageOperationGuard(object):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.parent.on_storage_operation_end()
 
+
 async def stats_collector(adaptor):
     while True:
         max_live_lock = 0
@@ -797,7 +841,21 @@ async def stats_collector(adaptor):
         max_lock_live_time.set(max_live_lock)
         await asyncio.sleep(5)
 
-async def live_lock_server(bind_to, port, release_all_timeout, password=None, max_payload=None, data_dir=None, shutdown_support=None):
+
+async def live_lock_server(bind_to, port, release_all_timeout, password=None,
+                           max_payload=None,
+                           data_dir=None,
+                           shutdown_support=None,
+                           tcp_keepalive_time=None,
+                           tcp_keepalive_interval=None,
+                           tcp_keepalive_probes=None,
+                           disable_dump_load=None,
+                           ):
+    # Sanitize values
+    tcp_keepalive_time = int(tcp_keepalive_time) if tcp_keepalive_time else None
+    tcp_keepalive_interval = int(tcp_keepalive_interval) if tcp_keepalive_interval else None
+    tcp_keepalive_probes = int(tcp_keepalive_probes) if tcp_keepalive_probes else None
+
     loop = asyncio.get_running_loop()
 
     try:
@@ -811,18 +869,27 @@ async def live_lock_server(bind_to, port, release_all_timeout, password=None, ma
         data_dir = os.getcwd()
 
     storage = InMemoryLockStorage(release_all_timeout=release_all_timeout)
-    logger.debug(f'Loading dump')
-    storage.load_dump()
-    stats = storage.stats()
+    if not disable_dump_load:
+        logger.debug(f'Loading dump')
+        storage.load_dump()
+        stats = storage.stats()
 
-    logger.info(f'Locks loaded from dump: {stats.get("lock_count", 0)}')
+        logger.info(f'Locks loaded from dump: {stats.get("lock_count", 0)}')
     logger.info(f'Starting live lock server at {bind_to}, {port}')
     logger.debug(f'release_all_timeout={release_all_timeout}')
     logger.debug(f'data_dir={data_dir}')
 
     adaptor = StorageAdaptor(storage)
 
-    server = await loop.create_server(lambda: LiveLockProtocol(adaptor=adaptor, password=password, max_payload=max_payload, shutdown_support=shutdown_support), bind_to, port)
+    server = await loop.create_server(lambda: LiveLockProtocol(adaptor=adaptor,
+                                                               password=password,
+                                                               max_payload=max_payload,
+                                                               shutdown_support=shutdown_support,
+                                                               tcp_keepalive_time=tcp_keepalive_time,
+                                                               tcp_keepalive_interval=tcp_keepalive_interval,
+                                                               tcp_keepalive_probes=tcp_keepalive_probes,
+                                                               ), bind_to, port)
+
     def exception_handler(loop, context):
         if not isinstance(context['exception'], SystemExit):
             raise context['exception']
@@ -837,8 +904,19 @@ async def live_lock_server(bind_to, port, release_all_timeout, password=None, ma
     stats_collector_task.cancel()
 
 
-def start(bind_to=DEFAULT_BIND_TO, port=None, release_all_timeout=None, password=None, max_payload=None, data_dir=None, shutdown_support=None):
-    logging.basicConfig(level=logging.DEBUG, format='%(name)s:[%(levelname)s]: %(message)s')
+def start(bind_to=DEFAULT_BIND_TO, port=None, release_all_timeout=None, password=None,
+          max_payload=None,
+          data_dir=None,
+          shutdown_support=None,
+          tcp_keepalive_time=None,
+          tcp_keepalive_interval=None,
+          tcp_keepalive_probes=None,
+          disable_dump_load=None,
+          ):
+    env_loglevel = get_settings(None, 'LIVELOCK_LOGLEVEL', DEFAULT_LOGLEVEL)
+    loglevel = getattr(logging, env_loglevel.upper())
+
+    logging.basicConfig(level=loglevel, format='%(name)s:[%(levelname)s]: %(message)s')
     prometheus_port = get_settings(port, 'LIVELOCK_PROMETHEUS_PORT', DEFAULT_PROMETHEUS_PORT)
     if prometheus_port:
         try:
@@ -847,8 +925,14 @@ def start(bind_to=DEFAULT_BIND_TO, port=None, release_all_timeout=None, password
             logger.critical(f'Wrong prometheus port {prometheus_port}')
             prometheus_port = None
     if prometheus_port:
-        logger.info(f'Starting prometheus metrics server at port {prometheus_port}')
-        start_http_server(prometheus_port)
+        try:
+            from prometheus_client import start_http_server
+        except ImportError:
+            logger.info('Prometheus port is set but prometheus_client not installed (pip install prometheus-client)')
+            prometheus_port = None
+        if prometheus_port:
+            logger.info(f'Starting prometheus metrics server at port {prometheus_port}')
+            start_http_server(prometheus_port)
 
     asyncio.run(live_lock_server(bind_to=get_settings(bind_to, DEFAULT_BIND_TO, 'LIVELOCK_BIND_TO'),
                                  port=get_settings(port, 'LIVELOCK_PORT', DEFAULT_LIVELOCK_SERVER_PORT),
@@ -857,4 +941,8 @@ def start(bind_to=DEFAULT_BIND_TO, port=None, release_all_timeout=None, password
                                  max_payload=get_settings(max_payload, 'LIVELOCK_MAX_PAYLOAD', DEFAULT_MAX_PAYLOAD),
                                  data_dir=get_settings(data_dir, 'LIVELOCK_DATA_DIR', None),
                                  shutdown_support=get_settings(shutdown_support, 'LIVELOCK_SHUTDOWN_SUPPORT', DEFAULT_SHUTDOWN_SUPPORT),
+                                 tcp_keepalive_time=get_settings(tcp_keepalive_time, 'LIVELOCK_TCP_KEEPALIVE_TIME', DEFAULT_TCP_KEEPALIVE_TIME),
+                                 tcp_keepalive_interval=get_settings(tcp_keepalive_interval, 'LIVELOCK_TCP_KEEPALIVE_INTERVAL', DEFAULT_TCP_KEEPALIVE_INTERVAL),
+                                 tcp_keepalive_probes=get_settings(tcp_keepalive_probes, 'LIVELOCK_TCP_KEEPALIVE_PROBES', DEFAULT_TCP_KEEPALIVE_PROBES),
+                                 disable_dump_load=get_settings(disable_dump_load, 'LIVELOCK_DISABLE_DUMP_LOAD', DEFAULT_DISABLE_DUMP_LOAD),
                                  ))
